@@ -84,9 +84,13 @@ ebl_symbol_type_name (ebl, symbol, buf, len)
 	res = stt_names[symbol];
       else
 	{
+	  char *ident;
+
 	  if (symbol >= STT_LOPROC && symbol <= STT_HIPROC)
 	    snprintf (buf, len, "LOPROC+%d", symbol - STT_LOPROC);
-	  else if (symbol == STT_GNU_IFUNC)
+	  else if (symbol == STT_GNU_IFUNC
+		   && (ident = elf_getident (ebl->elf, NULL)) != NULL
+		   && ident[EI_OSABI] == ELFOSABI_LINUX)
 	    return "GNU_IFUNC";
 	  else if (symbol >= STT_LOOS && symbol <= STT_HIOS)
 	    snprintf (buf, len, "LOOS+%d", symbol - STT_LOOS);
